/*******************************************************************************
 * Copyright 2015 - 2018 fmCreation
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald diese von der
 * Europischen Kommission genehmigt wurden - Folgeversionen der EUPL
 * ("Lizenz"); Sie drfen dieses Werk ausschlielich gem dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 *   https://progs.frankmuenster.de/eupl_v1-2_de
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften gefordert oder in
 * schriftlicher Form vereinbart, wird die unter der Lizenz verbreitete Software
 * "so wie sie ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet. Die sprachspezifischen
 * Genehmigungen und Beschrnkungen unter der Lizenz sind dem Lizenztext zu
 * entnehmen.
 ******************************************************************************/

package de.frankmuenster.mahoe.banks.aktien.assistant;

import java.math.BigDecimal;
import java.rmi.RemoteException;
import java.util.List;
import java.util.Locale;

import org.apache.commons.lang.StringUtils;

import de.frankmuenster.jameica.finanzen.FinanceException;
import de.frankmuenster.jameica.finanzen.boerseard.ArdStockInfoBean;
import de.frankmuenster.jameica.finanzen.boerseard.BoerseArdApi;
import de.frankmuenster.mahoe.MahoeSynchronizeBackend;
import de.frankmuenster.mahoe.assistant.AbstractAccountMigrator;
import de.frankmuenster.mahoe.banks.aktien.Aktien;
import de.frankmuenster.mahoe.utils.I18N;
import de.frankmuenster.mahoe.utils.Utils;
import de.willuhn.datasource.rmi.DBIterator;
import de.willuhn.jameica.hbci.Settings;
import de.willuhn.jameica.hbci.SynchronizeOptions;
import de.willuhn.jameica.hbci.rmi.HBCIDBService;
import de.willuhn.jameica.hbci.rmi.Konto;
import de.willuhn.jameica.hbci.rmi.Umsatz;
import de.willuhn.jameica.messaging.StatusBarMessage;
import de.willuhn.jameica.plugin.Plugin;
import de.willuhn.jameica.system.Application;
import de.willuhn.logging.Logger;
import de.willuhn.util.ApplicationException;

/**
 * Der Migrator fr das alte BSH Skript * @author Frank Mnster
 */
public class AktienAccountMigrator extends AbstractAccountMigrator {

    /**
     * Der alte Plugin Name
     */
    private static final String OLD_PLUGIN_NAME = "hibiscus.script.aktien"; //$NON-NLS-1$

    /**
     * Standardkonstruktor
     */
    public AktienAccountMigrator() {

        super();
    }

    /**
     * @see AbstractAccountMigrator#migrate(de.willuhn.jameica.plugin.Plugin)
     */
    @Override
    public void migrate(final Plugin plugin) throws ApplicationException {

        int countMigrated = 0;
        if (AktienAccountMigrator.OLD_PLUGIN_NAME.equals(plugin.getManifest().getName())) {
            Logger.info("Aktien-Migrator gestartet."); //$NON-NLS-1$
            try {
                final HBCIDBService hcbiService = Settings.getDBService();
                final DBIterator<Konto> accounts = hcbiService.createList(Konto.class);
                while (accounts.hasNext()) {
                    final Konto account = accounts.next();
                    if (Aktien.BLZ.equals(account.getBLZ())
                            && !MahoeSynchronizeBackend.BACKEND_NAME.equals(account.getBackendClass())) {
                        this.migrateAccount(account);
                        countMigrated++;
                    }
                }
            } catch (final RemoteException e) {
                Logger.error(I18N.tr("AktienAccountMigrator.searchAccountsFailed", e.getMessage()), e); //$NON-NLS-1$
            }
            if (countMigrated > 0) {
                final StatusBarMessage statusBarMsg = new StatusBarMessage(
                        I18N.tr("AktienAccountMigrator.migrationSuccessful"), //$NON-NLS-1$
                        StatusBarMessage.TYPE_SUCCESS);
                Application.getMessagingFactory().sendMessage(statusBarMsg);
            }
            // Plugin entfernen
            this.uninstallPlugin(plugin);
        }
    }

    /**
     * @see de.frankmuenster.mahoe.assistant.AbstractAccountMigrator#update(de.willuhn.jameica.hbci.rmi.Konto)
     */
    @Override
    public void update(final Konto account) throws ApplicationException {

        try {
            if (Aktien.BLZ.equals(account.getBLZ())) {

                final String oldVersion = account.getMeta(MahoeSynchronizeBackend.KONTO_VERSION, "0"); //$NON-NLS-1$
                switch (oldVersion) { // NOPMD
                case "0": //$NON-NLS-1$
                    Logger.info(I18N.tr(AbstractAccountMigrator.I18N_UPDATE_TO_VERSION, "1")); //$NON-NLS-1$
                    Utils.removePdfMeta(account);
                    account.setMeta(MahoeSynchronizeBackend.KONTO_VERSION, "1"); //$NON-NLS-1$
                    break;
                default:
                    break;
                }
            }
        } catch (final RemoteException e) {
            Logger.error(I18N.tr("AbstractAccountMigrator.updatedFailed"), e); //$NON-NLS-1$
            throw new ApplicationException(I18N.tr("AbstractAccountMigrator.updatedFailedMsg", e.getMessage()), e); //$NON-NLS-1$
        }

    }

    /**
     * Liefert entweder den einen Treffer oder <b>NULL</b>
     *
     * @param results
     *            die Liste mit den Treffern
     * @param name
     *            der Name des Wertpapiers
     * @return das bereinstimmende Wertpapier oder <b>NULL</b>
     */
    private ArdStockInfoBean findstock(final List<ArdStockInfoBean> results, final String name) {

        ArdStockInfoBean result = null;
        if (results.size() == 1) { // NOPMD
            result = results.get(0);
        } else {
            for (final ArdStockInfoBean stock : results) {
                Logger.debug(stock.getName() + "-" + name); //$NON-NLS-1$
                if (StringUtils.startsWithIgnoreCase(stock.getName(), name)) {
                    result = stock;
                    break;
                }
            }
        }
        return result;
    }

    /**
     * Wir mssen nur die Backend-Engine ndern
     *
     * @param konto
     *            das zu migrierende Konto
     * @throws ApplicationException
     */
    private void migrateAccount(final Konto konto) throws ApplicationException {

        try {
            konto.setBackendClass("de.frankmuenster.mahoe.MahoeSynchronizeBackend");//$NON-NLS-1$
            konto.setFlags(Konto.FLAG_NONE);
            konto.store();

            // Sync-Options setzen, nur zur Sicherheit
            final SynchronizeOptions sync = new SynchronizeOptions(konto);
            sync.setAutoSaldo(false);
            sync.setSyncAuslandsUeberweisungen(false);
            sync.setSyncKontoauszuege(true);
            sync.setSyncOffline(false);
            sync.setSyncSaldo(false);
            sync.setSyncSepaDauerauftraege(false);
            sync.setSyncSepaLastschriften(false);

            @SuppressWarnings("unchecked")
            final DBIterator<Umsatz> stocks = konto.getUmsaetze();
            while (stocks.hasNext()) {
                final Umsatz stock = stocks.next();
                this.updateStock(stock);
            }
        } catch (final RemoteException e) {
            Logger.error(I18N.tr("BshAccountMigrator.migrationFailedLog", e.getMessage()), e); //$NON-NLS-1$
            throw new ApplicationException(I18N.tr("BshAccountMigrator.migrationFailedMsg", e.getMessage()), e); //$NON-NLS-1$
        }
    }

    /**
     * Update one stock to the new version
     *
     * @param stock
     * @return <i>true</i> wenn das Wertpapier aktualisiert werden konnte,
     *         <i>false</i> sonst
     */
    private boolean updateStock(final Umsatz stock) {

        boolean updateSuccessful = false;
        try {
            String name = stock.getZweck();
            if (name.contains("...")) { //$NON-NLS-1$
                name = name.substring(0, name.indexOf("...")); //$NON-NLS-1$
            } else if (name.contains(" (")) { //$NON-NLS-1$
                name = name.substring(0, name.indexOf(" (")); //$NON-NLS-1$
            }
            final List<ArdStockInfoBean> results = BoerseArdApi.getInstance().getSearchResults(name);
            final ArdStockInfoBean result = this.findstock(results, name);
            if (result != null) {
                final String anzahlString = stock.getKommentar();
                String anzahl = "0.0"; //$NON-NLS-1$
                if (anzahlString.toLowerCase(Locale.GERMAN).contains("zahl=")) { //$NON-NLS-1$
                    anzahl = anzahlString.substring(7);
                }
                anzahl = anzahl.replace('.', ',');
                stock.setGegenkontoName(result.getName());
                stock.setArt(result.getType());
                stock.setZweck(I18N.tr("AktienSettingsController.amount") + anzahl); //$NON-NLS-1$
                // Das waren die wichtigen Dinge -> speichern
                stock.store();
                stock.setMeta(Aktien.STOCK_ID, result.getId());
                // Wertpaier aktualisiern
                BoerseArdApi.getInstance().getActualValue(result);
                stock.setKommentar(I18N.tr("AktienSettingsController.exchange") + result.getExchange()); //$NON-NLS-1$
                stock.setDatum(result.getDate());
                stock.setValuta(result.getDate());
                stock.setBetrag(result.getActValue().doubleValue());
                // Gesamtwert
                String amountStr = stock.getZweck();
                amountStr = amountStr.substring(amountStr.indexOf('=') + 1);
                amountStr = StringUtils.replace(amountStr, ".", StringUtils.EMPTY); //$NON-NLS-1$
                amountStr = StringUtils.replace(amountStr, ",", "."); //$NON-NLS-1$//$NON-NLS-2$
                final BigDecimal amount = new BigDecimal(amountStr);
                final BigDecimal total = amount.multiply(result.getActValue());
                stock.setSaldo(total.doubleValue());
                stock.store();
                stock.setMeta(Aktien.STOCK_EXCHANGE, result.getExchange());
                updateSuccessful = true;
            }
        } catch (RemoteException | FinanceException | ApplicationException e) {
            // Nur loggen
            Logger.error("OK, dann eben nicht. " + e.getLocalizedMessage(), e); //$NON-NLS-1$

        } finally {
            if (!updateSuccessful) {
                try {
                    stock.setGegenkontoName(I18N.tr("AktienAccountMigrator.recreateStock"));//$NON-NLS-1$
                    stock.store();
                } catch (RemoteException | ApplicationException e) {
                    // Nur loggen
                    Logger.error(e.getLocalizedMessage(), e);
                }

            }
        }
        return updateSuccessful;
    }
}
