
package de.frankmuenster.mahoe.banks.bsh.json.accountdetails;

import java.util.List;

import com.fasterxml.jackson.annotation.JsonInclude;
import com.fasterxml.jackson.annotation.JsonProperty;
import com.fasterxml.jackson.annotation.JsonPropertyOrder;

/**
 * VertragDetails - CoreData (Vertragsdaten)
 *
 * @author Frank Mnster
 */
@JsonInclude(JsonInclude.Include.NON_NULL)
@JsonPropertyOrder({ "bsn", "vtn", "role", "owners", "subsidycharge" })
public class CoreData {

    /**
     * Bausparnummer (BSN)
     */
    @JsonProperty("bsn")
    private String bsn;

    /**
     * Vertragsinhaber
     */
    @JsonProperty("owners")
    private List<String> owners;

    /**
     * Die Rolle im Vertrag
     */
    @JsonProperty("role")
    private String role;

    /**
     * Begnstigte Aufwendungen
     */
    @JsonProperty("subsidycharge")
    private String subsidycharge;

    /**
     * Die VTN
     */
    @JsonProperty("vtn")
    private String vtn;

    /**
     * Standardkonstruktor
     */
    public CoreData() {
	// Absichtlich leer
    }

    /**
     * Liefert die Bausparnummer
     *
     * @return die Bausparnummer
     */
    @JsonProperty("bsn")
    public String getBsn() {

	return this.bsn;
    }

    /**
     * Liefert die Vertragsinhaber
     *
     * @return ein List emit den Vertragsinhaber
     */
    @JsonProperty("owners")
    public List<String> getOwners() {

	return this.owners;
    }

    /**
     * Liefert die Rolle im Vertrag
     *
     * @return die Rolle im Vertrag
     */
    @JsonProperty("role")
    public String getRole() {

	return this.role;
    }

    /**
     * Liefert die beg&uuml;nstigten Aufwendungen
     *
     * @return die beg&uuml;nstigten Aufwendungen
     */
    @JsonProperty("subsidycharge")
    public String getSubsidycharge() {

	return this.subsidycharge;
    }

    /**
     * Liefert die VTN
     *
     * @return die VTN
     */
    @JsonProperty("vtn")
    public String getVtn() {

	return this.vtn;
    }

}
