
package de.frankmuenster.mahoe.banks.bsh.json.accountdetails;

import java.math.BigDecimal;
import java.util.List;

import com.fasterxml.jackson.annotation.JsonInclude;
import com.fasterxml.jackson.annotation.JsonProperty;
import com.fasterxml.jackson.annotation.JsonPropertyOrder;
import com.fasterxml.jackson.databind.annotation.JsonDeserialize;

import de.frankmuenster.mahoe.banks.bsh.json.deserializer.SimpleMoneyDeserializer;

/**
 * Vertragdetails - Subsidy
 *
 * @author Frank Mnster
 */
@JsonInclude(JsonInclude.Include.NON_NULL)
@JsonPropertyOrder({ "sum", "details" })
public class Subsidy {

    /**
     * Die Summe aller Prmien
     */
    @JsonProperty("sum")
    @JsonDeserialize(using = SimpleMoneyDeserializer.class)
    private BigDecimal sum;

    /**
     * Liste mit den Details der Prmien
     */
    @JsonProperty("details")
    private List<SubsidyDetail> details;

    /**
     * Standardkonstruktor
     */
    public Subsidy() {
	// Absichtlich leer
    }

    /**
     * Liefert die Liste mit den Details der Prmien.
     *
     * @return die Liste mit den Details.
     */
    @JsonProperty("details")
    public List<SubsidyDetail> getDetails() {

	return this.details;
    }

    /**
     * Liefert die Summe aller Prmien.
     *
     * @return die Summe aller Prmien.
     */
    @JsonProperty("sum")
    public BigDecimal getSum() {

	return this.sum;
    }

}
