/*******************************************************************************
 * Copyright 2015 - 2018 fmCreation
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald diese von der
 * Europischen Kommission genehmigt wurden - Folgeversionen der EUPL
 * ("Lizenz"); Sie drfen dieses Werk ausschlielich gem dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 *   https://progs.frankmuenster.de/eupl_v1-2_de
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften gefordert oder in
 * schriftlicher Form vereinbart, wird die unter der Lizenz verbreitete Software
 * "so wie sie ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet. Die sprachspezifischen
 * Genehmigungen und Beschrnkungen unter der Lizenz sind dem Lizenztext zu
 * entnehmen.
 ******************************************************************************/

package de.frankmuenster.mahoe.banks.santander.jobs;

import java.rmi.RemoteException;
import java.util.Collections;
import java.util.List;

import org.apache.commons.lang.NullArgumentException;

import de.frankmuenster.mahoe.MahoeSynchronizeBackend;
import de.frankmuenster.mahoe.banks.santander.clearing.ClearingEntry;
import de.frankmuenster.mahoe.banks.santander.web.AbstractSantanderWebSite;
import de.frankmuenster.mahoe.banks.santander.web.SantanderBankMessages;
import de.frankmuenster.mahoe.banks.santander.web.SantanderWebAccountInformation;
import de.frankmuenster.mahoe.banks.santander.web.SantanderWebSiteKreditkarte;
import de.frankmuenster.mahoe.banks.santander.web.SantanderWebSiteUtils;
import de.frankmuenster.mahoe.jobs.AbstractMahoeSynchronizeJobKontoauszug;
import de.frankmuenster.mahoe.utils.I18N;
import de.frankmuenster.mahoe.utils.TransactionSaver;
import de.frankmuenster.mahoe.web.WebSiteException;
import de.willuhn.jameica.hbci.rmi.KontoType;
import de.willuhn.jameica.hbci.rmi.Umsatz;
import de.willuhn.jameica.messaging.StatusBarMessage;
import de.willuhn.jameica.system.Application;
import de.willuhn.logging.Logger;
import de.willuhn.util.ApplicationException;

/**
 * Hier wird die eigentliche Arbeit gemacht.
 *
 * @author Frank Mnster
 */
public class SantanderSynchronizeJobKontoauszug
        extends AbstractMahoeSynchronizeJobKontoauszug<SantanderWebAccountInformation, SantanderBankMessages> {

    /**
     * Standartkonstruktor
     */
    public SantanderSynchronizeJobKontoauszug() {

        super();
    }

    /**
     * @see de.frankmuenster.mahoe.jobs.AbstractMahoeSynchronizeJobKontoauszug#getWebSite()
     */
    @Override
    protected void getWebSite() throws WebSiteException, ApplicationException {

        this.webSite = AbstractSantanderWebSite.getInstance(this.konto);
    }

    /**
     * @see de.frankmuenster.mahoe.jobs.AbstractMahoeSynchronizeJobKontoauszug#handleTransactions()
     */
    @Override
    protected void handleTransactions() throws WebSiteException, ApplicationException {

        try {
            if (KontoType.find(this.konto.getAccountType()).equals(KontoType.KREDITKARTE)) {
                this.monitor.setStatusText(I18N.tr("SantanderSynchronizeJobKontoauszug.getPreBooked")); //$NON-NLS-1$
                final List<Umsatz> preBooked = this.webSite.getPreBookedTransactions();
                // log transaction if log level is debug
                this.debugLogTransactions(preBooked);

                // Speicher der vorgemerkten Umstze
                this.monitor.setStatusText(I18N.tr("SantanderSynchronizeJobKontoauszug.savingPreBooked")); //$NON-NLS-1$
                Collections.reverse(preBooked);
                final int preBookedCount = TransactionSaver.savePreBooked(this.konto, preBooked, false);
                this.monitor.setStatusText(
                        I18N.tr("SantanderSynchronizeJobKontoauszug.savedPreBooked", Integer.toString(preBookedCount))); //$NON-NLS-1$
                this.monitor.setPercentComplete(50);
                Logger.info(
                        I18N.tr("SantanderSynchronizeJobKontoauszug.savedPreBooked", Integer.toString(preBookedCount))); //$NON-NLS-1$
            }
        } catch (final RemoteException e) {
            Logger.error(I18N.tr("SantanderSynchronizeJobKontoauszug.getPreBookedFailed"), e); //$NON-NLS-1$
        }
        this.monitor.setStatusText(I18N.tr("SynchronizeJobKontoauszug.getBooked", //$NON-NLS-1$
                this.accountInfo.getLastTransactionDateAsString()));
        final List<Umsatz> booked = this.webSite.getBookedTransactions();
        // log transaction if log level is debug
        this.debugLogTransactions(booked);

        // Speichere gebuchte Umstze
        this.monitor.setStatusText(I18N.tr("SynchronizeJobKontoauszug.savingBooked")); //$NON-NLS-1$
        Collections.reverse(booked);
        final int bookedCount = TransactionSaver.saveBooked(this.konto, booked, false);
        this.monitor.setStatusText(I18N.tr("SynchronizeJobKontoauszug.savedBooked", Integer.toString(bookedCount))); //$NON-NLS-1$
        this.monitor.setPercentComplete(50);
        Logger.info(I18N.tr("SynchronizeJobKontoauszug.savedBooked", Integer.toString(bookedCount))); //$NON-NLS-1$
        // berprfe, ob eine Ausgleichsbuchung angelegt werden soll
        this.handleClearing();
    }

    /**
     * @see de.frankmuenster.mahoe.jobs.AbstractMahoeSynchronizeJobKontoauszug#login()
     */
    @Override
    protected void login() throws WebSiteException, ApplicationException {

        SantanderWebSiteUtils.login(this.webSite, this.konto, this.kundennummer);

    }

    /**
     * berprft, ob eine Ausgleichsbuchung angelegt werden soll.
     */
    private void handleClearing() {

        if (!(this.webSite instanceof SantanderWebSiteKreditkarte)) {
            return;
        }
        try {
            if ("JA".equals(this.konto.getMeta(MahoeSynchronizeBackend.CREATE_CLEARING, "NEIN"))) { //$NON-NLS-1$ //$NON-NLS-2$
                final ClearingEntry clearingEntry = ((SantanderWebSiteKreditkarte) this.webSite).getClearingEntry();
                clearingEntry.setKonto(this.konto);
                if (!clearingEntry.exists()) {
                    if (clearingEntry.saveEntry()) { // NOPMD
                        // AvoidDeeplyNestedIfStmts
                        final String msg = I18N.tr("SantanderSynchronizeJobKontoauszug.clearingTransactionCreated"); //$NON-NLS-1$
                        Logger.info(msg);
                        this.monitor.setStatusText(msg);
                        final StatusBarMessage msgUser = new StatusBarMessage(msg, StatusBarMessage.TYPE_INFO);
                        Application.getMessagingFactory().sendMessage(msgUser);
                    } else {
                        final String msg = I18N.tr("SantanderSynchronizeJobKontoauszug.clearingAmountIsZero"); //$NON-NLS-1$
                        Logger.info(msg);
                        this.monitor.setStatusText(msg);
                    }
                }
            }
        } catch (final NullArgumentException e) {
            final String msg = I18N.tr("SantanderSynchronizeJobKontoauszug.clearingAccountIsMissing"); //$NON-NLS-1$
            Logger.error(msg, e);
            this.monitor.setStatusText(msg);
            final StatusBarMessage msgUser = new StatusBarMessage(msg, StatusBarMessage.TYPE_ERROR);
            Application.getMessagingFactory().sendMessage(msgUser);
        } catch (final RemoteException e) {
            Logger.error(I18N.tr("SantanderSynchronizeJobKontoauszug.createClearingTransactionFailedLog"), e); //$NON-NLS-1$
            final String msg = I18N.tr("SantanderSynchronizeJobKontoauszug.createClearingTransactionFailedMsg"); //$NON-NLS-1$
            this.monitor.setStatusText(msg);
            final StatusBarMessage msgUser = new StatusBarMessage(msg, StatusBarMessage.TYPE_ERROR);
            Application.getMessagingFactory().sendMessage(msgUser);
        }
        // 2 Sekunden warten, damit der Benutzer eine Chance hat, die
        // Meldung zu
        // lesen
        try {
            Thread.sleep(2000);
        } catch (final InterruptedException e) {
            Thread.currentThread().interrupt();
        }
    }
}
