
package de.frankmuenster.mahoe.gui.controller;

import java.io.File;
import java.io.FileInputStream;
import java.io.InputStream;

import de.frankmuenster.mahoe.MahoeMain;
import de.frankmuenster.mahoe.utils.I18N;
import de.willuhn.io.FileFinder;
import de.willuhn.jameica.gui.AbstractControl;
import de.willuhn.jameica.gui.AbstractView;
import de.willuhn.jameica.gui.Part;
import de.willuhn.jameica.gui.parts.FormTextPart;
import de.willuhn.jameica.plugin.AbstractPlugin;
import de.willuhn.jameica.plugin.Manifest;
import de.willuhn.jameica.system.Application;
import de.willuhn.jameica.util.InfoReader;
import de.willuhn.logging.Logger;

/**
 * Controller fuer den Lizenzinformationen Dialog Basiert auf dem
 * {@link de.willuhn.jameica.gui.internal.controller#LincenseControl}
 *
 * @author willuhn/Frank Mnster
 * @version 1.0
 */
public class LicenseController extends AbstractControl {

    /**
     * HMTL Line break
     */
    private static final String HTML_BR = "<br />"; //$NON-NLS-1$

    /**
     * FormTextPart um dem Text schn formatieren zu knnen
     */
    private transient FormTextPart libList;

    /**
     * Standrdart Konstruktor liefert ein LincenceControl object
     *
     * @param view
     *            die View, fuer die dieser LicenceControl zustaendig ist
     */
    public LicenseController(final AbstractView view) {
	super(view);
    }

    /**
     * Liefert eine Liste mit allen direkt von jameica.qrscanner verwendeten
     * Komponenten.
     *
     * @return Liste der Komponenten.
     */
    public Part getLibList() {

	if (this.libList == null) {

	    final AbstractPlugin plugin = Application.getPluginLoader().getPlugin(MahoeMain.class);

	    final StringBuilder buffer = new StringBuilder(
		    "<form><p><span color=\"header\" font=\"header\">Mahoe</span></p>"); //$NON-NLS-1$

	    Manifest manifest = null;
	    manifest = plugin.getManifest();
	    if (manifest != null) {
		buffer.append("<p>"); //$NON-NLS-1$
		buffer.append(manifest.getDescription());
		buffer.append(LicenseController.HTML_BR).append(manifest.getHomepage());
		buffer.append(LicenseController.HTML_BR).append(manifest.getLicense());
		buffer.append("</p>"); //$NON-NLS-1$
	    }

	    final String path = plugin.getManifest().getPluginDir();

	    final FileFinder finder = new FileFinder(new File(path + "/lib")); //$NON-NLS-1$
	    finder.matches(".*?info\\.xml$"); //$NON-NLS-1$
	    final File[] infos = finder.findRecursive();
	    for (int i = 0; i < infos.length; ++i) {
		if (!infos[i].isFile() || !infos[i].canRead()) {
		    Logger.warn(I18N.tr("LicenseController.infoFileReadError", infos[i].getName())); //$NON-NLS-1$
		    continue;
		}

		try {
		    final InputStream infoXml = new FileInputStream( // NOPMD
			    infos[i]);
		    final InfoReader infoReader = new InfoReader( // NOPMD
			    infoXml);
		    infoXml.close();
		    buffer.append("<p><b>").append(infoReader.getName()) //$NON-NLS-1$
			    .append("</b>"); //$NON-NLS-1$
		    buffer.append(LicenseController.HTML_BR).append(infoReader.getDescription());
		    buffer.append(LicenseController.HTML_BR).append(infoReader.getUrl());
		    buffer.append(LicenseController.HTML_BR).append(infoReader.getLicense());
		    buffer.append("</p>"); //$NON-NLS-1$

		} catch (final Exception e) { // NOPMD
		    Logger.error(I18N.tr("LicenseController.infoFileParseError", infos[0].getName()), e); //$NON-NLS-1$
		}
	    }
	    buffer.append("</form>"); //$NON-NLS-1$

	    this.libList = new FormTextPart(buffer.toString());
	}
	return this.libList;
    }
}
