/*******************************************************************************
 * Copyright 2015 - 2018 fmCreation
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald diese von der
 * Europischen Kommission genehmigt wurden - Folgeversionen der EUPL
 * ("Lizenz"); Sie drfen dieses Werk ausschlielich gem dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 *   https://progs.frankmuenster.de/eupl_v1-2_de
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften gefordert oder in
 * schriftlicher Form vereinbart, wird die unter der Lizenz verbreitete Software
 * "so wie sie ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet. Die sprachspezifischen
 * Genehmigungen und Beschrnkungen unter der Lizenz sind dem Lizenztext zu
 * entnehmen.
 ******************************************************************************/

package de.frankmuenster.mahoe.gui.controller;

import java.rmi.RemoteException;

import org.apache.commons.lang.StringUtils;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Event;

import de.frankmuenster.mahoe.MahoeSynchronizeBackend;
import de.frankmuenster.mahoe.banks.targobank.TargoBank;
import de.frankmuenster.mahoe.banks.targobank.assistant.TargoBankCreateInfo;
import de.frankmuenster.mahoe.utils.I18N;
import de.willuhn.jameica.hbci.rmi.Konto;
import de.willuhn.jameica.messaging.StatusBarMessage;
import de.willuhn.jameica.system.Application;
import de.willuhn.logging.Logger;

/**
 * Controller for the settings view of the Targo Bank.
 *
 * @author Frank Mnster
 */
public class TargoBankSettingsController extends AbstractSettingsController<TargoBankCreateInfo> {

    /**
     * Standard constructor
     */
    public TargoBankSettingsController() {

        super(TargoBank.BLZ, new TargoBankCreateInfo());

        // set account information
        this.ktoInfo.setComment(StringUtils.EMPTY);
        this.ktoInfo.setDescription(StringUtils.EMPTY);
        this.ktoInfo.setAccountType(null);
        this.ktoInfo.setLogin(StringUtils.EMPTY);
        this.ktoInfo.setOwner(StringUtils.EMPTY);
        this.ktoInfo.setOwner(StringUtils.EMPTY);

    }

    /**
     * @see de.frankmuenster.mahoe.gui.controller.SettingsController#speichern ()
     */
    @Override
    public void speichern() {

        if (this.statementsMaxDaysBack.getValue() == null) {
            this.ktoInfo.setStatementsMaxDaysBack(-1);
        } else {
            this.ktoInfo.setStatementsMaxDaysBack((int) this.statementsMaxDaysBack.getValue());
        }
        final Boolean testsOK = this.validateData();
        if (testsOK) {

            final Konto konto = (Konto) this.getKonto().getValue();
            try {
                final String downloadPeriod = Integer.toString(this.ktoInfo.getStatementsMaxDaysBack());
                konto.setMeta(MahoeSynchronizeBackend.ACC_STATEM_MAX_DAYS_BACK, downloadPeriod);
                final String msg = I18N.tr("SettingsController.settingsSaved"); //$NON-NLS-1$
                Logger.info(msg);
                final StatusBarMessage msgUser = new StatusBarMessage(msg, StatusBarMessage.TYPE_INFO);
                Application.getMessagingFactory().sendMessage(msgUser);
            } catch (final RemoteException e) {
                Logger.error(I18N.tr("SettingsController.saveSettingsFailedLog"), e); //$NON-NLS-1$
                final String msg = I18N.tr("SettingsController.saveSettingsFailedMsg"); //$NON-NLS-1$
                final StatusBarMessage msgUser = new StatusBarMessage(msg, StatusBarMessage.TYPE_ERROR);
                Application.getMessagingFactory().sendMessage(msgUser);
            }

        }

    }

    @Override
    protected void handleAccountChanged(final Event event) {

        if (event.type != SWT.Selection) {
            return;
        }

        // Kontotyp ermitteln
        final Konto konto = (Konto) this.account.getValue();
        // Not used fields
        this.ktoInfo.setComment(StringUtils.EMPTY);
        this.ktoInfo.setDescription(StringUtils.EMPTY);
        this.ktoInfo.setLogin(StringUtils.EMPTY);
        this.ktoInfo.setOwner(StringUtils.EMPTY);
        if (konto != null) {
            try {
                final int statmMaxDaysBack = Integer
                        .parseInt(konto.getMeta(MahoeSynchronizeBackend.ACC_STATEM_MAX_DAYS_BACK, "90")); //$NON-NLS-1$
                this.ktoInfo.setStatementsMaxDaysBack(statmMaxDaysBack);
            } catch (final RemoteException e) {
                final String msg = I18N.tr("SettingsController.readAccountSettingsFailed"); //$NON-NLS-1$
                Logger.error(msg, e);
                final StatusBarMessage msgUser = new StatusBarMessage(msg, StatusBarMessage.TYPE_ERROR);
                Application.getMessagingFactory().sendMessage(msgUser);
            }
        }

        this.getSavePDF().setValue(this.supportsStatements());
        if (this.supportsStatements()) {
            this.getStatementMaxDaysBack().setValue(this.ktoInfo.getStatementsMaxDaysBack());
            this.getStatementMaxDaysBack().enable();
        } else {
            this.getStatementMaxDaysBack().setValue(0);
            this.getStatementMaxDaysBack().disable();
        }

    }

    /**
     * Validates if settings are OK
     *
     * @return <i>true</i> in case all settings are OK, <i>false</i> else
     */
    private boolean validateData() {

        boolean testsOK = true;

        if (this.ktoInfo.getStatementsMaxDaysBack() < 0) {
            final String msg = I18N.tr("TargoBankSettingsController.downloadRangeNotGivenMsg"); // NOPMD //$NON-NLS-1$
            final StatusBarMessage msgUser = new StatusBarMessage(msg, StatusBarMessage.TYPE_ERROR);
            Application.getMessagingFactory().sendMessage(msgUser);
            this.statementsMaxDaysBack.setComment(I18N.tr("TargoBankSettingsController.downloadRangeNotGivenComment")); //$NON-NLS-1$
            testsOK = false;
        }
        return testsOK;
    }

}
