/*******************************************************************************
 * Copyright 2015 - 2019 fmCreation
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald diese von der
 * Europischen Kommission genehmigt wurden - Folgeversionen der EUPL
 * ("Lizenz"); Sie drfen dieses Werk ausschlielich gem dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 *   https://progs.frankmuenster.de/eupl_v1-2_de
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften gefordert oder in
 * schriftlicher Form vereinbart, wird die unter der Lizenz verbreitete Software
 * "so wie sie ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet. Die sprachspezifischen
 * Genehmigungen und Beschrnkungen unter der Lizenz sind dem Lizenztext zu
 * entnehmen.
 ******************************************************************************/

package de.frankmuenster.mahoe.utils;

import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.nio.file.Files;
import java.time.Duration;

import org.apache.commons.io.IOUtils;
import org.openqa.selenium.TimeoutException;
import org.openqa.selenium.WebDriver;
import org.openqa.selenium.WebElement;
import org.openqa.selenium.support.ui.FluentWait;

import com.gargoylesoftware.htmlunit.html.HtmlAnchor;

import de.frankmuenster.mahoe.web.SeleniumWebSite;
import de.willuhn.logging.Logger;
import de.willuhn.util.ApplicationException;

/**
 * Utility class for web methods
 *
 * @author Frank Mnster
 */
public final class WebUtils {

    /**
     * HTML tag href
     */
    public static final String TAG_HREF = "href"; //$NON-NLS-1$

    /**
     * HTML tag tr
     */
    public static final String TAG_TD = "td"; //$NON-NLS-1$

    /**
     * HTML tag tr
     */
    public static final String TAG_TR = "tr"; //$NON-NLS-1$

    /**
     * Read the account statement PDF from an HTMLUnit anchor into a byte array.
     *
     * @param link
     *            the HMTLUnit anchor to the PDF
     * @return the PDF as byte array
     */
    public static byte[] getPdfAsByteArray(final HtmlAnchor link) {

        byte[] data = null;
        try (InputStream inputStream = link.click().getWebResponse().getContentAsStream()) {

            data = IOUtils.toByteArray(inputStream);

        } catch (final IOException exception) {
            Logger.error(exception.getMessage());
        }

        return data;

    }

    /**
     * Read the account statement PDF into a byte array.
     *
     * @param website
     *            the selenium page to load the PDF document from
     * @param linkElement
     *            the link on the page to the PDF
     * @return the PDF as byte array
     */
    public static byte[] getSeleniumPdfAsByteArray(final SeleniumWebSite website, final WebElement linkElement) {

        final WebDriver driver = website.getDriver();

        byte[] data = null;
        String link = linkElement.getAttribute(WebUtils.TAG_HREF);

        link = link.substring(29, link.length() - 2);
        final String[] args = link.split(","); //$NON-NLS-1$
        final String expectedFileName = args[2].substring(1, args[2].length() - 1);

        Logger.debug("Downloading file: " + expectedFileName); //$NON-NLS-1$

        linkElement.click();
        try {

            final FluentWait<WebDriver> wait = new FluentWait<>(driver).withTimeout(Duration.ofSeconds(60))
                    .pollingEvery(Duration.ofMillis(200));
            final File fileToCheck = website.getDownloadsDirectory().resolve(expectedFileName).toFile();

            // Wait until download is complete
            wait.until((final WebDriver wd) -> fileToCheck.exists());
            // read file in memory
            data = Files.readAllBytes(fileToCheck.toPath());
            // delete file again
            Files.delete(fileToCheck.toPath());

        } catch (final ApplicationException | IOException | TimeoutException e) {
            Logger.error(I18N.tr("WebUtils.errorDownloadAccountSheet", expectedFileName), e); //$NON-NLS-1$
        }

        return data;

    }

    /**
     * private class constructor
     */
    private WebUtils() {

        // intentionally left blank
    }

}
